package com.perforce.hive.plugin.local;

import java.io.InputStream;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import com.perforce.hive.api.DefaultEngine;
import com.perforce.hive.api.Loader;
import com.perforce.hive.api.artifact.ArtifactInterface;
import com.perforce.hive.api.artifact.ArtifactType;
import com.perforce.hive.api.artifact.ResultInterface;
import com.perforce.hive.api.artifact.SummaryInterface;
import com.perforce.hive.api.repository.RepoInterface;
import com.perforce.hive.api.search.ResultDetailed;
import com.perforce.p4java.core.IDepot;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.server.IOptionsServer;

import ro.fortsoft.pf4j.Extension;
import ro.fortsoft.pf4j.Plugin;
import ro.fortsoft.pf4j.PluginWrapper;
import spark.Request;
import spark.Response;
import spark.Spark;

public class UnknownPlugin extends Plugin {

	private static String id;

	public UnknownPlugin(PluginWrapper wrapper) {
		super(wrapper);
		id = wrapper.getPluginId();
	}

	@Extension
	public static class LocalLoader implements Loader {

		@Override
		public ArtifactType getType() {
			ArtifactType type = new ArtifactType(id, "UNKNOWN");
			type.setTitle("Unknown Artifact");
			return type;
		}

		@Override
		public void loadImage(String endpoint) {
			Spark.get(endpoint, (request, response) -> {
				InputStream image = UnknownPlugin.class.getResourceAsStream("/images/unknown.png");
				HttpServletResponse raw = DefaultEngine.rawImage(image, response);
				image.close();
				return raw;
			});
		}

		@Override
		public ArtifactInterface getArtifact(Request request, Response response) throws Exception {
			return new LocalArtifact(request, response);
		}

		@Override
		public ArtifactInterface getProxy(IOptionsServer p4, String repoID) throws Exception {
			return new LocalArtifact(p4, repoID);
		}

		@Override
		public SummaryInterface getSummary(IOptionsServer p4, List<IFileSpec> summary) {
			// Not implemented
			return null;
		}

		@Override
		public ResultInterface getResult(ResultDetailed result) {
			return new LocalResult(result);
		}

		@Override
		public RepoInterface createRepository(ArtifactType type) {
			return new LocalRepository(type);
		}

		@Override
		public RepoInterface createRepository(Request request) {
			return new LocalRepository(request);
		}

		@Override
		public RepoInterface createRepository(IDepot depot) {
			return new LocalRepository(depot);
		}

		@Override
		public Class<?> getRepositoryClass() {
			return LocalRepository.class;
		}
	}

}
